function keys.ui.copier.deactivate_buttons()
	keys.ui.copier.clear_clipboard_button.active = "NO"
	keys.ui.copier.copy_to_clipboard_button.active = "NO"
	keys.ui.copier.copy_from_clipboard_button.active = "NO"
	keys.ui.copier.clear_key_members_button.active = "NO"
end

function keys.ui.copier.activate_buttons()
	keys.ui.copier.clear_clipboard_button.active = "YES"
	keys.ui.copier.copy_to_clipboard_button.active = "YES"
	keys.ui.copier.copy_from_clipboard_button.active = "YES"
	keys.ui.copier.clear_key_members_button.active = "YES"
end

function keys.ui.copier.generate_copier_tab(copier_tab)
	keys.clipboard_members = {}
	keys.ui.copier.selected_key_info = nil
	keys.ui.copier.selected_key_members = nil

	-- a_hbox elements (1st column)
	local key_list = iup.list {"<select key>", dropdown = "YES", expand = "HORIZONTAL"}
	local key_list_i = 2
	for _,ki in pairs(keys.infos) do
		local color_str = ""
		local owner_key_id_str = ki.owner_key_id
		if ki.owner_key_id == false then
			owner_key_id_str = ""
			color_str = "\127FFD559"
		else
			if ki.active then
				color_str = "\1275BB349"
			else
				color_str = "\127dddddd"
			end
		end
		key_list[key_list_i] = color_str..ki.desc.." \127aaaaaa(k_id="..ki.id..",o_id="..owner_key_id_str..")"
		key_list_i = key_list_i + 1
  end
	keys.ui.copier.key_member_list = iup.list {expand="YES",readonly="YES"}
	function key_list:action (text, item, state)
		while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
		keys.ui.copier.selected_key_members = nil
		keys.ui.copier.selected_key_info = nil
		local key_id
		key_id = string.match(text, "k_id=([0-9]+)")
		for _,ki in pairs(keys.infos) do
			if ki.id == key_id then
				keys.ui.copier.selected_key_info = ki
				keys.ui.copier.selected_key_members = {}
				local key_member_list_i = 1
				for _,m in pairs(ki.members) do
					table.insert(keys.ui.copier.selected_key_members, m)
					keys.ui.copier.key_member_list[key_member_list_i] = m.name
					key_member_list_i = key_member_list_i + 1
				end
			end
		end
	end
	keys.ui.copier.clear_key_members_button = iup.button{title="Clear key members",expand="HORIZONTAL"}
	function keys.ui.copier.clear_key_members_button:action()
		if keys.ui.copier.selected_key_info == nil then return end
		local is_owner_key = (keys.ui.copier.selected_key_info.owner_key_id == false)
		if is_owner_key then
			keys.print("Whoops, you'd better not to try clear owner members :)")
			return
		end
		-- set UI to progress state
		keys.ui.copier.deactivate_buttons()
		while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
		keys.ui.copier.key_member_list[1] = "Reloading"
		-- revoke keys
		local my_name = GetPlayerName()
		for _,m in pairs(keys.ui.copier.selected_key_members) do
			if m.name == my_name then
				keys.print("not revoking from you...")
			else
				Keychain.revokekey(m.name, keys.ui.copier.selected_key_info.id)
			end
			-- revoking keys won't update instantly, so ..
			-- continue process after all keys are processed ..
			-- .. check OnEvent()
		end
	end
	-- b_hbox elements (2nd column)
	keys.ui.copier.copy_to_clipboard_button   = iup.button{title="->",expand="HORIZONTAL"}
	keys.ui.copier.copy_from_clipboard_button = iup.button{title="<-",expand="HORIZONTAL"}

	-- c_hbox elements (3rd column)
	local clipboard_member_list = iup.list {expand="YES"}
	keys.ui.copier.clear_clipboard_button = iup.button{title="Clear clipboard", expand="HORIZONTAL"}	
	function keys.ui.copier.clear_clipboard_button:action()
		while(clipboard_member_list[1]) do clipboard_member_list[1] = nil end -- clear list
		keys.clipboard_members = {}
	end

	-- copy button actions
	function keys.ui.copier.copy_to_clipboard_button:action()
		if keys.ui.copier.selected_key_info == nil then return end

		for _,m in pairs(keys.ui.copier.selected_key_members) do
			table.insert(keys.clipboard_members, m)
		end
		while(clipboard_member_list[1]) do clipboard_member_list[1] = nil end -- clear list
		local clipboard_member_list_i = 1
		for _,m in pairs(keys.clipboard_members) do
			clipboard_member_list[clipboard_member_list_i] = m.name
			clipboard_member_list_i = clipboard_member_list_i + 1
		end
	end
	function keys.ui.copier.copy_from_clipboard_button:action()
		if keys.ui.copier.selected_key_info == nil then return end
		-- set UI to progress state
		keys.ui.copier.deactivate_buttons()
		while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
		keys.ui.copier.key_member_list[1] = "Reloading"
		-- give keys
		for _,m in pairs(keys.clipboard_members) do
			Keychain.giveuserkey(m.name, keys.ui.copier.selected_key_info.id)
			-- giving keys won't update instantly, so ..
			-- continue process after all keys are given ..
			-- .. check OnEvent()
		end
	end

	-- copier tab ui
	local a_vbox = iup.vbox {iup.label{title="Key Name"}, 
													 key_list, 
													 iup.fill{},
													 iup.label{title="Members"},
													 keys.ui.copier.key_member_list, 
													 keys.ui.copier.clear_key_members_button}
	local b_vbox = iup.vbox {iup.fill{}, iup.label{title="Copy user keys"},keys.ui.copier.copy_to_clipboard_button, keys.ui.copier.copy_from_clipboard_button, iup.fill{}, alignment="ACENTER", gap=15}
	local c_vbox = iup.vbox {iup.label{title="Clipboard"},
													 clipboard_member_list, 
													 keys.ui.copier.clear_clipboard_button}
	local copier_box = iup.hbox {a_vbox, b_vbox, c_vbox}

	copier_tab:append(copier_box)
end
